<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Client;
use Illuminate\Http\Request;

class ClientsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $per_page = 25;
        if (! empty($request->get('per_page'))) {
            $per_page = $request->get('per_page');
        }
        $data = Client::when($request->get('name'), function ($query) use ($request) {
            return $query->where('name', 'like', '%'.$request->get('name').'%');
        })
            ->when($request->get('email'), function ($query) use ($request) {
                return $query->where('email', 'like', '%'.$request->get('email').'%');
            })
            ->when($request->get('phone'), function ($query) use ($request) {
                return $query->where('phone', 'like', '%'.$request->get('phone').'%');
            })
            ->orderBy('id', 'DESC')
            ->paginate($per_page);

        return $data;
    }

    public function getAll()
    {
        $clients = Client::get();

        return $clients;
    }

    /**
     * Store a newly created resource in storage.
     *
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'last_name' => 'required',
        ]);
        Client::create($request->all());

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $data = Client::findOrFail($id);

        return $data;
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name' => 'required',
            'last_name' => 'required',
        ]);
        Client::find($id)->update($request->all());

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $data = Client::findOrFail($id);
        $data->deleted = 1;
        $data->save();
        //$data->delete();
    }
}
