<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Libraries\Common;
use App\Models\AdditionalJewel;
use App\Models\AdditionalWatch;
use App\Models\Client;
use App\Models\Invoice;
use App\Models\Setting;
use App\Models\Worker;
use Barryvdh\DomPDF\Facade\Pdf;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;

class InvoicesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $per_page = 25;
        if (! empty($request->get('per_page'))) {
            $per_page = $request->get('per_page');
        }
        $data = Invoice::when($request->get('invoice_number'), function ($query) use ($request) {
            return $query->where('invoice_number', 'like', '%'.$request->get('invoice_number').'%');
        })
            ->when($request->get('client'), function ($query) use ($request) {
                return $query->join('clients', 'clients.id', '=', 'invoices.client_id')
                    ->where('clients.id', $request->get('client'));
            })
            ->with(['client', 'additional_jewel', 'additional_watch'])
            ->orderBy('invoices.invoice_number', 'DESC')
            ->select([
                'invoices.id',
                'invoices.invoice_number',
                'invoices.date',
                'invoices.total',
                'invoices.client_id',
                'invoices.pdf',
                'invoices.pdf_signed',
                DB::raw("'0' as loading"),
                DB::raw("'0' as loading_signed_pdf"),
            ])
            ->paginate($per_page);

        return $data;
    }

    /**
     * Store a newly created resource in storage.
     *
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'title' => 'required',
            'total' => 'required',
        ]);
        DB::beginTransaction();

        try {
            if ($invoice = Invoice::create($request->all())) {

                $in = Setting::where('key', 'inv_counter')->firstOrFail();
                $invoice->invoice_number = $in->value;
                $in->value = $in->value + 1;
                $in->save();
                $invoice->save();

                $client_data = $request->get('client');
                $client = null;
                if (! isset($client_data['id'])) {
                    $client = new Client;
                } else {
                    $client = Client::find($client_data['id']);
                }
                if (isset($client_data['name'])) {
                    $client->name = $client_data['name'];
                }
                if (isset($client_data['last_name'])) {
                    $client->last_name = $client_data['last_name'];
                }
                if (isset($client_data['address'])) {
                    $client->address = $client_data['address'];
                }
                if (isset($client_data['city'])) {
                    $client->city = $client_data['city'];
                }
                if (isset($client_data['zip'])) {
                    $client->zip = $client_data['zip'];
                }
                if (isset($client_data['phone'])) {
                    $client->phone = $client_data['phone'];
                }
                if (isset($client_data['state'])) {
                    $client->state = $client_data['state'];
                }
                if (isset($client_data['email'])) {
                    $client->email = $client_data['email'];
                }
                $client->save();
                $invoice->client_id = $client->id;

                $additional_jewel = $request->get('additional_jewel');
                $additional_watch = $request->get('additional_watch');

                if ($invoice->type == 'jewelry' && ! empty($additional_jewel)) {
                    $aJewel = new AdditionalJewel;
                    if (! empty($additional_jewel['gender'])) {
                        $aJewel->gender = $additional_jewel['gender'];
                    }
                    if (! empty($additional_jewel['length'])) {
                        $aJewel->length = $additional_jewel['length'];
                    }
                    if (! empty($additional_jewel['color_gold'])) {
                        $aJewel->color_gold = $additional_jewel['color_gold'];
                    }
                    if (! empty($additional_jewel['weight'])) {
                        $aJewel->weight = $additional_jewel['weight'];
                    }
                    if (! empty($additional_jewel['metal_purity'])) {
                        $aJewel->metal_purity = $additional_jewel['metal_purity'];
                    }
                    if (! empty($additional_jewel['total_diamond_weight'])) {
                        $aJewel->total_diamond_weight = $additional_jewel['total_diamond_weight'];
                    }
                    if (! empty($additional_jewel['clarity_grade'])) {
                        $aJewel->clarity_grade = $additional_jewel['clarity_grade'];
                    }
                    if (! empty($additional_jewel['color_grade'])) {
                        $aJewel->color_grade = $additional_jewel['color_grade'];
                    }
                    $aJewel->invoice_id = $invoice->id;
                    $aJewel->save();
                } elseif ($invoice->type == 'watches' && ! empty($additional_watch)) {
                    $aWatch = new AdditionalWatch;
                    //if (!empty($additional_watch['gender'])) $aWatch->gender = $additional_watch['gender'];
                    if (! empty($additional_watch['color_gold'])) {
                        $aWatch->color_gold = $additional_watch['color_gold'];
                    }
                    if (! empty($additional_watch['metal_purity'])) {
                        $aWatch->metal_purity = $additional_watch['metal_purity'];
                    }
                    if (! empty($additional_watch['total_diamond_weight'])) {
                        $aWatch->total_diamond_weight = $additional_watch['total_diamond_weight'];
                    }
                    if (! empty($additional_watch['clarity_grade'])) {
                        $aWatch->clarity_grade = $additional_watch['clarity_grade'];
                    }
                    if (! empty($additional_watch['color_grade'])) {
                        $aWatch->color_grade = $additional_watch['color_grade'];
                    }
                    if (! empty($additional_watch['custom'])) {
                        $aWatch->custom = $additional_watch['custom'];
                    }
                    if (! empty($additional_watch['style_number'])) {
                        $aWatch->style_number = $additional_watch['style_number'];
                    }
                    if (! empty($additional_watch['serial_number'])) {
                        $aWatch->serial_number = $additional_watch['serial_number'];
                    }
                    $aWatch->invoice_id = $invoice->id;
                    $aWatch->save();
                }

                $invoice->date = date('Y-m-d H:i:s');
                $invoice->save();
                //$this->generatePdf($invoice->id);
            }

            DB::commit();
        } catch (\Exception $e) {
            DB::rollBack();
        }

    }

    public function upload(Request $request, $id)
    {
        print_r(Request::file('files'));
        exit();
        $imageName = time().'.'.$request->file->getClientOriginalName();

        $request->file->move(public_path('images'), $imageName);

        return response()->json(['success' => $imageName]);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $data = Invoice::with(['client', 'additional_watch', 'additional_jewel'])
            ->select([
                'invoices.*',
                DB::raw("'0' as loading"),
                DB::raw("'0' as loading_signed_pdf"),
            ])
            ->findOrFail($id);

        return $data;
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'title' => 'required',
            'total' => 'required',
        ]);

        DB::beginTransaction();

        try {

            $invoice = Invoice::find($id);

            if ($invoice->image != $request->get('image')) {
                $name = explode('/images/', $invoice->image);
                if (! empty($name[1])) {
                    if (file_exists(public_path('images/').$name[1])) {
                        @unlink(public_path('images/').$name[1]);
                    }
                }

            }
            if ($invoice->update($request->all())) {

                if (file_exists(public_path('pdf/').$invoice->pdf)) {
                    @unlink(public_path('pdf/').$invoice->pdf);
                }
                if (file_exists(public_path('pdf/').$invoice->pdf_signed)) {
                    @unlink(public_path('pdf/').$invoice->pdf_signed);
                }
                $invoice->pdf = '';
                $invoice->pdf_signed = '';

                $client_data = $request->get('client');
                $client = '';
                if (! isset($client_data['id'])) {
                    $client = new Client;
                } else {
                    $client = Client::find($client_data['id']);
                }
                if (isset($client_data['name'])) {
                    $client->name = $client_data['name'];
                }
                if (isset($client_data['last_name'])) {
                    $client->last_name = $client_data['last_name'];
                }
                if (isset($client_data['address'])) {
                    $client->address = $client_data['address'];
                }
                if (isset($client_data['city'])) {
                    $client->city = $client_data['city'];
                }
                if (isset($client_data['zip'])) {
                    $client->zip = $client_data['zip'];
                }
                if (isset($client_data['phone'])) {
                    $client->phone = $client_data['phone'];
                }
                if (isset($client_data['state'])) {
                    $client->state = $client_data['state'];
                }
                if (isset($client_data['email'])) {
                    $client->email = $client_data['email'];
                }
                $client->save();
                $invoice->client_id = $client->id;

                $invoice->save();

                AdditionalJewel::where('invoice_id', $invoice->id)->delete();
                AdditionalWatch::where('invoice_id', $invoice->id)->delete();

                $additional_jewel = $request->get('additional_jewel');
                $additional_watch = $request->get('additional_watch');

                if ($invoice->type == 'jewelry' && ! empty($additional_jewel)) {
                    $aJewel = new AdditionalJewel;
                    if (! empty($additional_jewel['gender'])) {
                        $aJewel->gender = $additional_jewel['gender'];
                    }
                    if (! empty($additional_jewel['length'])) {
                        $aJewel->length = $additional_jewel['length'];
                    }
                    if (! empty($additional_jewel['color_gold'])) {
                        $aJewel->color_gold = $additional_jewel['color_gold'];
                    }
                    if (! empty($additional_jewel['weight'])) {
                        $aJewel->weight = $additional_jewel['weight'];
                    }
                    if (! empty($additional_jewel['metal_purity'])) {
                        $aJewel->metal_purity = $additional_jewel['metal_purity'];
                    }
                    if (! empty($additional_jewel['total_diamond_weight'])) {
                        $aJewel->total_diamond_weight = $additional_jewel['total_diamond_weight'];
                    }
                    if (! empty($additional_jewel['clarity_grade'])) {
                        $aJewel->clarity_grade = $additional_jewel['clarity_grade'];
                    }
                    if (! empty($additional_jewel['color_grade'])) {
                        $aJewel->color_grade = $additional_jewel['color_grade'];
                    }
                    $aJewel->invoice_id = $invoice->id;
                    $aJewel->save();
                } elseif ($invoice->type == 'watches' && ! empty($additional_watch)) {
                    $aWatch = new AdditionalWatch;
                    //if (!empty($additional_watch['gender'])) $aWatch->gender = $additional_watch['gender'];
                    if (! empty($additional_watch['color_gold'])) {
                        $aWatch->color_gold = $additional_watch['color_gold'];
                    }
                    if (! empty($additional_watch['metal_purity'])) {
                        $aWatch->metal_purity = $additional_watch['metal_purity'];
                    }
                    if (! empty($additional_watch['total_diamond_weight'])) {
                        $aWatch->total_diamond_weight = $additional_watch['total_diamond_weight'];
                    }
                    if (! empty($additional_watch['clarity_grade'])) {
                        $aWatch->clarity_grade = $additional_watch['clarity_grade'];
                    }
                    if (! empty($additional_watch['color_grade'])) {
                        $aWatch->color_grade = $additional_watch['color_grade'];
                    }
                    if (! empty($additional_watch['custom'])) {
                        $aWatch->custom = $additional_watch['custom'];
                    }
                    if (! empty($additional_watch['style_number'])) {
                        $aWatch->style_number = $additional_watch['style_number'];
                    }
                    if (! empty($additional_watch['serial_number'])) {
                        $aWatch->serial_number = $additional_watch['serial_number'];
                    }
                    $aWatch->invoice_id = $invoice->id;
                    $aWatch->save();
                }
            }
            $this->generatePdf($id);
            DB::commit();
        } catch (\Exception $e) {
            \Log::info($e->getMessage());
            DB::rollBack();
        }

    }

    public function uploadImage(Request $request)
    {
        $ret = [
            'result' => true,
            'name' => '',
        ];

        $image = $request->file('img');

        $originalName = $image->getClientOriginalName();

        $name = Str::slug(uniqid().'_'.$originalName, '_');

        $filename = public_path('images/').$name.'.png';

        $image->move(public_path('images/'), $filename);
        $ret['name'] = '/images/'.$name.'.png';
        $ret['result'] = true;

        return $ret;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $data = Invoice::findOrFail($id);
        $data->deleted = 1;
        $data->save();
        //$data->delete();
    }

    public function send(Request $request)
    {
        try {
            $invoice = new \App\Mail\Invoice();

            $item = $request->get('item');
            $pdf = $request->get('pdf');
            $pdf_signed = $request->get('pdf_signed');

            if (! empty($pdf_signed)) {
                $invoice->pdf = $pdf_signed;
            } else {
                $invoice->pdf = $pdf;
            }

            $invoice->images = $request->get('images');

            Mail::to($request->get('email'))->send($invoice);
        } catch (Exception $ex) {
            Log::warning('Showing send email error for user: '.$email['to_address'].'. The id in Email table: '.$email['id']);
        }
    }

    public function total($id)
    {
        $invoice = Invoice::findOrFail($id);
        $tax = Setting::where('key', 'tax')->firstOrFail();
        $sub_total = Common::getSubTotal($invoice);
        $tax_cal = $sub_total * $tax->value / 100;
        if ($invoice->payment_method == 'check') {
            $total = $sub_total;
        } else {
            $total = round($sub_total + $tax_cal, 2);
        }
        $invoice->total = $total;
        $invoice->save();

        return $total;
    }

    public function generatePdf($id)
    {
        $invoice = Invoice::with(['client', 'additional_jewel', 'additional_watch'])->findOrFail($id);
        $list = Setting::get();
        $settings = [];
        foreach ($list as $item) {
            $settings[$item->key] = $item->value;
        }
        $manager = Worker::where('is_main', 'y')->first();
        $logourl = public_path('/img/logopdf.png');
        $escaleUrl = public_path('/img/platinum-scale.png');
        $logo = $this->convertToBase64($logourl);
        $imageUrl = $invoice->image ? public_path($invoice->image) : null;
        $image1Url = $invoice->image ? public_path($invoice->image1) : null;
        $image = $imageUrl ? $this->convertToBase64($imageUrl) : null;
        $image1 = $image1Url ? $this->convertToBase64($image1Url) : null;
        $escale = $this->convertToBase64($escaleUrl);

        $data = [
            'invoice' => $invoice,
            'settings' => $settings,
            'manager_signature' => $manager->signature,
            'logo' => $logo,
            'image' => $image,
            'image1' => $image1,
            'escale' => $escale,

        ];
        $pdf = PDF::loadView('pdf', $data);
        $time = time();

        $pdf->setPaper('A4', 'portrait');

        $pdf->getDomPDF()->getOptions()->set([
            'margin_top' => 0,
            'margin_bottom' => 0,

        ]);
        $pdf->save('pdf/'.Str::slug($invoice->client->name).'-'.$invoice->invoice_number.'-'.date('d-m-Y').'('.$time.')'.'.pdf');
        $invoice->pdf = Str::slug($invoice->client->name).'-'.$invoice->invoice_number.'-'.date('d-m-Y').'('.$time.')'.'.pdf';
        $invoice->save();

        return $invoice;

    }

    public function convertToBase64($imagen)
    {
        try {
            $path = $imagen;
            $type = pathinfo($path, PATHINFO_EXTENSION);
            $data = file_get_contents($path);
            $imagen = 'data:image/'.$type.';base64,'.base64_encode($data);

            return $imagen;
        } catch (\Throwable $th) {
            return null;
        }
    }
}
