<?php

namespace App\Http\Controllers;

use App\Models\Invoice;
use App\Models\Setting;
use App\Models\Worker;
use Barryvdh\DomPDF\Facade\Pdf;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use Twilio\Rest\Client;

class AppController extends Controller
{
    protected string $smsLog;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('app');
    }

    public function index1()
    {
        return 'app';
    }

    public function generatePdf($id)
    {
        $invoice = Invoice::with(['client', 'additional_jewel', 'additional_watch'])->findOrFail($id);
        $list = Setting::get();
        $settings = [];
        foreach ($list as $item) {
            $settings[$item->key] = $item->value;
        }
        $manager = Worker::where('is_main', 'y')->first();
        $logourl = public_path('/img/logopdf.png');
        $escaleUrl = public_path('/img/platinum-scale.png');
        $logo = $this->convertToBase64($logourl);
        $imageUrl = $invoice->image ? public_path($invoice->image) : null;
        $image1Url = $invoice->image ? public_path($invoice->image1) : null;
        $image = $imageUrl ? $this->convertToBase64($imageUrl) : null;
        $image1 = $image1Url ? $this->convertToBase64($image1Url) : null;
        $escale = $this->convertToBase64($escaleUrl);

        $data = [
            'invoice' => $invoice,
            'settings' => $settings,
            'manager_signature' => $manager->signature,
            'logo' => $logo,
            'image' => $image,
            'image1' => $image1,
            'escale' => $escale,

        ];
        $pdf = PDF::loadView('pdf', $data);
        $time = time();

        $pdf->setPaper('A4', 'portrait');

        $pdf->getDomPDF()->getOptions()->set([
            'margin_top' => 0,
            'margin_bottom' => 0,

        ]);
        $pdf->save('pdf/'.Str::slug($invoice->client->name).'-'.$invoice->invoice_number.'-'.date('d-m-Y').'('.$time.')'.'.pdf', 'pdf');
        $invoice->pdf = Str::slug($invoice->client->name).'-'.$invoice->invoice_number.'-'.date('d-m-Y').'('.$time.')'.'.pdf';
        $invoice->save();

        return $invoice;

        // return view('pdf', $data);
    }

    public function singPdf(Request $request)
    {
        $id = $request->get('id');
        $img = $request->get('img');
        $invoice = Invoice::with(['client', 'additional_jewel', 'additional_watch'])->findOrFail($id);
        $list = Setting::get();
        $settings = [];
        foreach ($list as $item) {
            $settings[$item->key] = $item->value;
        }
        $manager = Worker::where('is_main', 'y')->first();

        $logourl = public_path('/img/logopdf.png');
        $escaleUrl = public_path('/img/platinum-scale.png');
        $logo = $this->convertToBase64($logourl);
        $imageUrl = $invoice->image ? public_path($invoice->image) : null;
        $image1Url = $invoice->image ? public_path($invoice->image1) : null;
        $image = $imageUrl ? $this->convertToBase64($imageUrl) : null;
        $image1 = $image1Url ? $this->convertToBase64($image1Url) : null;
        $escale = $this->convertToBase64($escaleUrl);
        $signatureUrl = $request->has('img') ? public_path($request->get('img')) : null;
        $signature = $this->convertToBase64($signatureUrl);

        $data = [
            'invoice' => $invoice,
            'settings' => $settings,
            'manager_signature' => $manager->signature,
            'logo' => $logo,
            'image' => $image,
            'image1' => $image1,
            'escale' => $escale,
            'signature' => $img,

        ];

        $pdf = PDF::loadView('pdf', $data);
        $time = time();
        $pdf->setPaper('a4')->setOption('margin-bottom', 0)->save('pdf/'.Str::slug($invoice->client->name).'-'.$invoice->invoice_number.'-'.date('d-m-Y').'('.$time.')'.'-signed.pdf', 'local');
        $invoice->pdf_signed = Str::slug($invoice->client->name).'-'.$invoice->invoice_number.'-'.date('d-m-Y').'('.$time.')'.'-signed.pdf';
        $invoice->save();

        return $invoice;
        //return view('pdf', $data);
    }

    public function generatePdfv($id)
    {
        $invoice = Invoice::with(['client', 'worker', 'vehicle', 'bills', 'parts', 'repairs'])->findOrFail($id);
        $list = Setting::get();
        $settings = [];
        foreach ($list as $item) {
            $settings[$item->key] = $item->value;
        }
        $billsDetail = [];
        $manager = Worker::where('is_main', 'y')->first();
        $tax = Setting::where('key', 'tax')->firstOrFail();
        $amount = 0;
        foreach ($invoice->bills as $bill) {
            foreach ($bill->bill_details as $detail) {
                $amount += $detail->quantity * $detail->price;
                $billsDetail[] = $detail;
            }
        }
        foreach ($invoice->parts as $part) {
            $amount += $part->quantity * $part->price;
        }
        $cant_p = $amount;
        $cant_r = 0;
        foreach ($invoice->repairs as $repair) {
            $amount += $repair->labor * $repair->time;
            $cant_r += $repair->labor * $repair->time;
        }
        $amount += $invoice->sublet_other;
        $amount += $invoice->shop_supplies;
        $amount += $invoice->fees;
        $taxc = 0;
        if ($invoice->payment_method != 'check') {
            $taxc = round(($amount * $tax->value / 100), 2);
            $total = round($amount + ($amount * $tax->value / 100), 2);
        } else {
            $total = round($amount, 2);
        }

        $data = [
            'invoice' => $invoice,
            'settings' => $settings,
            'billsDetail' => $billsDetail,
            'tax' => $taxc,
            'parts' => round($cant_p, 2),
            'repairs' => round($cant_r, 2),
            'sub_total' => $amount,
            'total' => $total,
            'manager_signature' => $manager->signature,
            'img' => '',

        ];
        //$pdf = PDF::loadView('pdf', $data);
        //	$pdf->stream($invoice->invoice_number.'.pdf');
        return view('pdf', $data);
    }

    public function sendClientSms(Request $request)
    {
        // Your Account SID and Auth Token from twilio.com/console
        $user = Auth::user();
        $pdf_url = '';
        $to_number = $request->input('phone');
        $message = $request->input('smsMessage');
        $pdf = $request->input('pdf');
        $pdf_signed = $request->input('pdf_signed');
        if (isset($pdf_signed)) {
            $pdf_url = url('pdf/'.$pdf_signed);
        } else {
            $pdf_url = url('pdf/'.$pdf);
        }

        try {
            $result = $this->sendSms($to_number, $message, $pdf_url);
            if ($result == 'ok') {
                return response()->json('ok', 200);
            } else {
                return response()->json($result, 400);
            }
        } catch (Exception $e) {

            return response()->json($e->getMessage(), 400);
        }
    }

    public function sendSms($number, $message, $url)
    {
        // Your Account SID and Auth Token from twilio.com/console
        $account_sid = $_ENV['TWILIO_ACCOUNT_SID'];
        $auth_token = $_ENV['TWILIO_AUTH_TOKEN'];
        $message = $message;
        $images = $url;
        $mediaUrl = [];
        $mediaUrl[] = $url;
        $twilio_number = $_ENV['TWILIO_NUMBER'];

        try {
            $client = new Client($account_sid, $auth_token);

            $client->messages->create(
                // Where to send a text message (your cell phone?)
                $number,
                [
                    'from' => $twilio_number,
                    'body' => $message,
                    'mediaUrl' => $mediaUrl,
                    'applicationSid' => 'test sms',
                ]
            );

            return 'ok';
        } catch (Exception $e) {

            return $e->getMessage();
        }
    }

    public function getSmsEnv()
    {
        return json_encode(config('app.apraisal_config')['sms_client']);
    }

    public function convertToBase64($imagen)
    {
        try {
            $path = $imagen;
            $type = pathinfo($path, PATHINFO_EXTENSION);
            $data = file_get_contents($path);
            $imagen = 'data:image/'.$type.';base64,'.base64_encode($data);

            return $imagen;
        } catch (\Throwable $th) {
            return null;
        }
    }
}
